'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const LoggerBroker_1 = require("../universal/LoggerBroker");
const pandora_service_logger_1 = require("pandora-service-logger");
const pandora_metrics_1 = require("pandora-metrics");
const GlobalConfigProcessor_1 = require("../universal/GlobalConfigProcessor");
const debug = require('debug')('pandora:cluster:monitor');
/**
 * Class Monitor
 */
class BaseMonitor {
    constructor() {
        this.daemonLogger = LoggerBroker_1.getDaemonLogger();
        this.globalConfigProcesser = GlobalConfigProcessor_1.GlobalConfigProcessor.getInstance();
        this.globalConfig = this.globalConfigProcesser.getAllProperties();
        this.metricsCollectPeriodConfig = pandora_metrics_1.MetricsCollectPeriodConfig.getInstance();
    }
    /**
     * Start Monitor
     * @return {Promise<void>}
     */
    async start() {
        // start logger rotator serivce
        debug('start a monitor logger rotator service');
        const loggerRotator = new pandora_service_logger_1.LoggerRotator({
            logger: this.daemonLogger
        });
        await loggerRotator.startService();
        // set global reporter interval
        this.metricsCollectPeriodConfig.configGlobalPeriod(this.globalConfig['reporterInterval']);
        // start metrics server
        debug('start a metrics server');
        this.server = new pandora_metrics_1.MetricsActuatorServer({
            config: this.globalConfig['actuator'],
            logger: this.daemonLogger,
            metricsManager: new this.globalConfig['metricsManager']()
        });
        this.startMetrics();
        this.startMetricsReporter();
        this.daemonLogger.info('monitor started');
    }
    startMetrics() {
        // register some default metrics
        let metricsManager = this.server.getMetricsManager();
        metricsManager.register('system', pandora_metrics_1.MetricName.build('system').setLevel(pandora_metrics_1.MetricLevel.MAJOR), new pandora_metrics_1.CpuUsageGaugeSet(this.metricsCollectPeriodConfig.getCachedTimeForLevel(pandora_metrics_1.MetricLevel.MAJOR)));
        metricsManager.register('system', pandora_metrics_1.MetricName.build('system').setLevel(pandora_metrics_1.MetricLevel.TRIVIAL), new pandora_metrics_1.NetTrafficGaugeSet(this.metricsCollectPeriodConfig.getCachedTimeForLevel(pandora_metrics_1.MetricLevel.TRIVIAL)));
        metricsManager.register('system', pandora_metrics_1.MetricName.build('system').setLevel(pandora_metrics_1.MetricLevel.TRIVIAL), new pandora_metrics_1.SystemMemoryGaugeSet(this.metricsCollectPeriodConfig.getCachedTimeForLevel(pandora_metrics_1.MetricLevel.TRIVIAL)));
        metricsManager.register('system', pandora_metrics_1.MetricName.build('system').setLevel(pandora_metrics_1.MetricLevel.MAJOR), new pandora_metrics_1.SystemLoadGaugeSet(this.metricsCollectPeriodConfig.getCachedTimeForLevel(pandora_metrics_1.MetricLevel.MAJOR)));
        metricsManager.register('system', pandora_metrics_1.MetricName.build('system').setLevel(pandora_metrics_1.MetricLevel.TRIVIAL), new pandora_metrics_1.DiskStatGaugeSet(this.metricsCollectPeriodConfig.getCachedTimeForLevel(pandora_metrics_1.MetricLevel.TRIVIAL)));
    }
    startMetricsReporter() {
        debug('start a metrics reporter');
        for (let reporterName in this.globalConfig['reporter']) {
            const reporterObj = this.globalConfig['reporter'][reporterName];
            if (reporterObj['enabled']) {
                let reporterIns = new reporterObj['target'](this.server, reporterObj.initConfig || {});
                reporterIns.start(this.globalConfig['reporterInterval']);
                this.daemonLogger.info(`${reporterName} reporter started`);
            }
        }
    }
    /**
     * @return {Promise<void>}
     */
    async stop() {
        await this.server.destroy();
    }
}
exports.BaseMonitor = BaseMonitor;
//# sourceMappingURL=Monitor.js.map